#
#  $Id: dtkplugins.py,v 1.8 2001/12/25 06:55:41 rob Exp $
#
#  Copyright 1999-2001 Rob Tillotson <rob@pyrite.org>
#  All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee or royalty is
#  hereby granted, provided that the above copyright notice appear in
#  all copies and that both the copyright notice and this permission
#  notice appear in supporting documentation or portions thereof,
#  including modifications, that you you make.
#
#  THE AUTHOR ROB TILLOTSON DISCLAIMS ALL WARRANTIES WITH REGARD TO
#  THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
#  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
#  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
#  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
#  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE!
#
"""base types of plugins, etc.
"""

__version__ = '$Id: dtkplugins.py,v 1.8 2001/12/25 06:55:41 rob Exp $'

__copyright__ = 'Copyright 1999-2001 Rob Tillotson <rob@pyrite.org>'


import sys, os, urlparse
from formatter import NullWriter
import dtkmain
from doc_database import DocWriteStream
import urllib

import plugin

# *******************************************************************
# Callback/API object for plugins
# *******************************************************************
class DTKCallback(plugin.Callback):
    def __init__(self, *a, **kw):
	apply(plugin.Callback.__init__, (self,)+a, kw)
    

# *******************************************************************
# Loader stuff
# *******************************************************************
#PLUGINS = [
#    'plugin_HTML', 'plugin_TaggedText', 'plugin_Text',
#    'plugin_RichReader', 'plugin_TealDoc',
#    'plugin_ztxtoutput',
#    'plugin_pdbinput', 'plugin_debugoutput',
#    ]


def load_all_plugins(callback=None):
    if globals().has_key('__file__'):
        path = os.path.split(__file__)[0]
        if path not in sys.path:
            sys.path.append(path)

        # figure out what plugins are here
        p = {}
        for fn, ext in map(os.path.splitext, os.listdir(path)):
            if ext in ['.py', '.pyc', '.pyo'] and fn[:7] == 'plugin_' and not p.has_key(fn):
                p[fn] = fn
        PLUGINS = p.keys()
    else:
        PLUGINS = []
        path = ''

    pl = [ RawParserPlugin(),
          BasicAssemblerPlugin(),
          URLStreamInputPlugin(),
          DocDBOutputPlugin() ]
    
    l = {}
    for p in pl: l[p.name] = p
    
    ldr = plugin.CallableLoader(callback)
    for p in ldr.load_plugins(PLUGINS, 'Plugin'):
	l[p.name] = p

    if path and path in sys.path:
        sys.path.remove(path)
        
    return l


# *******************************************************************
# Input plugin base class and raw plugin
# *******************************************************************

class DTKPlugin(plugin.PropertyMixin, plugin.CLIOptionMixin):
    links = []
    def __init__(self, type='plugin'):
	plugin.PropertyMixin.__init__(self,'Pyrite.Publisher.%s.%s' % (type, self.name))
	plugin.CLIOptionMixin.__init__(self, [])
	
	self._add_property('name', 'The name of this plugin', read_only=1)
	self._add_property('version', 'The version of this plugin', read_only=1)
	self._add_property('author', 'The author of this plugin', read_only=1)
	self._add_property('author_email', "The author's email address", read_only=1)
	self._add_property('description', 'A description of this plugin', read_only=1)

	self.chain = None
	self.next = None
	
    def get_supported_links(self):
	return self.links

    def set_priority(self, npri, input=None, output=None):
        l = []
        for pri, inp, outp in self.links:
            if inp == input and outp == output: pri = npri
            elif inp == input and output == None: pri = npri
            elif input == None and output == None: pri = npri
            l.append((pri, inp, outp))
        l.sort()
        self.links = l
        
    def open(self, chain, next, *a, **kw):
	self.chain = chain
	self.next = next
	return self

    def close(self):
	self.chain = None
	self.next = None
	


class ParserPlugin(DTKPlugin):
    name = ''
    version = dtkmain.VERSION
    author = 'Rob Tillotson'
    author_email = 'rob@pyrite.org'
    description = ''

    def __init__(self, *a, **kw):
	DTKPlugin.__init__(self, 'Parser')

    def feed(self, data): pass
    def eof(self): pass

class RawParserPlugin(ParserPlugin):
    name = 'RawText'
    description = 'Processes raw input.'

    links = [ (0, "application/octet-stream", "doc-assembler") ]

    def feed(self, data):
	self.next.send_literal_data(data)

# *******************************************************************
# Output plugins base class and basic doc output plugin
# *******************************************************************

class AssemblerPlugin(DTKPlugin):
    name = ''
    version = dtkmain.VERSION
    author = 'Rob Tillotson'
    author_email = 'rob@pyrite.org'
    description = ''

    def __init__(self, *a, **kw):
	DTKPlugin.__init__(self, 'Assembler')

    
    
    
class BasicDocWriter(NullWriter):
    def __init__(self, doc=sys.stdout):
	NullWriter.__init__(self)

	self.doc = doc
	self.list_depth = 0
        self.footnote_marker_format = '[%d]'
        self.features = []
        
    def has_feature(self, f):
        return f in self.features or self.doc.has_feature(f)
    
    def close(self):
	self.flush()
	self.doc.close()
	self.doc = None

    def set_bookmark(self, s):
	"""Set a bookmark at the current output position.
	"""
	self.flush()
	self.doc.bookmark(s)

    def set_annotation(self, text, t=''):
        self.flush()
        self.doc.annotate(text, t)

    def mark_footnote(self, number=0):
        self.send_literal_data(self.footnote_marker_format % number)
        
    def send_heading(self, text, level=1):
	self.send_literal_data(text)
	self.send_line_break()

    def set_title(self, title):
	self.doc.name = title

    # standard writer API
    def reset(self):
	self.list_depth = 0

    def send_paragraph(self, blankline=0):
	self.doc.write('\n'*blankline)

    def send_line_break(self):
	self.doc.write('\n')

    def send_hor_rule(self, *a, **kw):
	char = kw.get('char','*')[:1]
	if char == '-': count = 39
	else: count = 26
	brk = kw.get('break','none')
	if brk in ['before','both']: self.doc.write('\n')
	self.doc.write(char * count)
	if brk in ['after','both']: self.doc.write('\n')

    def send_literal_data(self, data):
	self.doc.write(data)

    def send_flowing_data(self, data):
	self.doc.write(data)

    def send_label_data(self, data):
	self.doc.write('  '*self.list_depth)
	self.doc.write(data)
	if data and data[-1] not in ' \n\r\t':
	    self.doc.write(' ')

    def new_alignment(self, align):
	pass

    def new_font(self, font):
	pass

    def new_margin(self, margin, level):
	if margin is None or level == 0:
	    self.list_depth = 0
	elif margin in ['ol', 'ul']:
	    self.list_depth = level - 1

    def new_spacing(self, spacing):
	pass

    def new_styles(self, styles):
	pass

    
class BasicAssemblerPlugin(AssemblerPlugin):
    name = 'BasicDoc'
    description = 'Assembles a standard text document.'

    links = [ (10, "doc-assembler", "doc-output") ]

    writerclass = BasicDocWriter

    def __init__(self, *a, **kw):
	apply(AssemblerPlugin.__init__, (self,)+a, kw)
	self.writer = None

	self._add_cli_option('footnote_marker_format', None, 'footnote-marker-format',
                             'Footnote marker format',
                             vtype="STR")
        self._add_property('footnote_marker_format', 'Footnote marker format')
        self.footnote_marker_format = '[%d]'
        
    def open(self, chain, next, *a, **kw):
	self.writer = self.writerclass(next)
        self.copyProperties(self.writer)
	return self.writer

    def close(self):
	self.writer.close()
	self.writer = None
    

#
#  input.
#

class InputPlugin(DTKPlugin):
    name = ''
    version = dtkmain.VERSION
    author = "Rob Tillotson"
    author_email = "rob@pyrite.org"
    description = ""

    def __init__(self, *a, **kw):
	DTKPlugin.__init__(self, "Input")

    def handles_filename(self, fn):
	return None
    
    def open_input(self, fn):
	"""Open a data source.
	"""
	return (None, None)

    def close_input(self):
	pass
    
    
class URLStreamInputPlugin(InputPlugin):
    name = 'URLStream'
    description = 'Gets input from a file or URL.'

    def __init__(self, *a, **kw):
	apply(InputPlugin.__init__, (self,)+a, kw)
	self.f = None
	
    def handles_filename(self, fn):
	return 1
    
    def open_input(self, fn):
	f = urllib.urlopen(fn)  # raises IOError if fails

	n, ext = os.path.splitext(os.path.basename(fn))
	if ext == '.gz':
	    f = GzipFile(filename="", mode="rb", fileobj=f)
	    n, ext = os.path.splitext(n)

	self.f = f

	mtype = f.info()["Content-Type"]

	self.type = mtype

	mtypes = [mtype]
	
	if mtype[:5] == "text/":
	    mtypes.append("application/octet-stream")

        if not n: # handle urls with no path
            up = urlparse.urlparse(fn)
            n = up[1]

	return mtypes, os.path.basename(n)
    
    def close_input(self):
	if self.f is not None: self.f.close()
	self.f = None

    def go(self, mimetype):
	while 1:
	    l = self.f.readline()
	    if not l: break
	    self.next.feed(l)
	self.next.eof()
	

class OutputPlugin(DTKPlugin):
    name = ''
    version = dtkmain.VERSION
    author = "Rob Tillotson"
    author_email = "rob@pyrite.org"
    description = ""

    def __init__(self, *a, **kw):
	DTKPlugin.__init__(self, "Output")


class DocDBOutputPlugin(OutputPlugin):
    name = 'DocOutput'
    description = 'Outputs to a Doc database.'

    links = [ (0, "doc-output", None),
              (0, "doc-only-output", None) ]
    # doc-only-output is for formats that will ever only work with
    # plain doc db (eg. richreader and teal); plain text formats and
    # things like HTML might one day be readable in zTXT or other formats
    # too, and thus can use doc-output.
    
    doc_version = 0
    
    def __init__(self, *a, **kw):
	OutputPlugin.__init__(self)

	self._add_property('title', 'The document title')
	self._add_cli_option('title', 't', 'title',
			     'Set the document title',
			     vtype='STR')
	self.title = ''
	
	self._add_property('backup', 'Set the backup bit', boolean=1)
	self._add_cli_option('backup', 'b', 'backup',
			     "Set the document's backup bit",
			     boolean=1)
	self.backup = 0
			     
	self._add_property('category', 'The document category')
	self._add_cli_option('category', 'c', 'category',
			     "Set the document's category",
			     vtype="NUM", func=int)
	self.category = 0
	
	self._add_property('creator', 'The document creator')
	self._add_cli_option('creator', 'C', 'creator',
			     "Set the document's creator ID",
			     vtype="STR")
	self.creator = 'REAd'
	
	self._add_property('type', 'The document type')
	self._add_cli_option('type', 'T', 'type',
			     "Set the document's type",
			     vtype="STR")
	self.type = 'TEXt'

	self.doc = None

	self._add_property('output_dir', 'Output directory')
        self._add_cli_option('output_dir', 'd', 'output-directory',
                             'Set the output directory',
                             vtype="STR")
        self.output_dir = '.'

        self._add_property('output_filename', 'Output filename')
        self._add_cli_option('output_filename', 'o', None,
                             'Set the output filename',
                             vtype="STR")
        self.output_filename = ''
        
        
    def open(self, chain, next, basename, *a, **kw):
        if self.output_filename: fn = self.output_filename
        else: fn = os.path.join(self.output_dir, basename+'.pdb')
        
	self.doc = DocWriteStream(fn,
                                  self.title and self.title or basename,
				  self.creator, self.type, self.backup and 0x0008 or 0,
				  self.doc_version, self.category, 1)
	return self.doc

    def close(self):
	self.doc = None
	
