
import dtkplugins, dtkmain

# This plugin is the first one to not fit into the usual 4-stage
# chain.  Its input interface is used by the pdbinput plugin to
# feed the contents of a doc directly, without reformatting but
# with metadata, through to a doc-output plugin.

class Plugin(dtkplugins.DTKPlugin):
    name = 'CopyDoc'
    description = 'Copies a doc database.'
    version = dtkmain.VERSION
    author = 'Rob Tillotson'
    author_email = 'rob@pyrite.org'
    
    links = [ (0, "DOC:raw", "doc-output") ]

    def open(self, chain, next, *a, **kw):
        # next is a docwritestream
        self.doc = next
        return self
    
    def close(self):
        self.doc.close()
        self.doc = None
        
    def feed_title(self, title):
        self.doc.name = title

    def feed_bookmark(self, s, pos):
        self.doc.bookmark(s, pos)

    def feed_annotation(self, text, t, pos):
        if self.doc.has_feature('annotate'):
            self.doc.annotate(text, t, pos)

    def feed(self, text):
        self.doc.write(text)

        
    
