#
#  $Id: plugin_ztxtoutput.py,v 1.5 2001/12/21 06:23:13 rob Exp $
#
#  Copyright 2001 Rob Tillotson <rob@pyrite.org>
#  All Rights Reserved
#
#  Permission to use, copy, modify, and distribute this software and
#  its documentation for any purpose and without fee or royalty is
#  hereby granted, provided that the above copyright notice appear in
#  all copies and that both the copyright notice and this permission
#  notice appear in supporting documentation or portions thereof,
#  including modifications, that you you make.
#
#  THE AUTHOR ROB TILLOTSON DISCLAIMS ALL WARRANTIES WITH REGARD TO
#  THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
#  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
#  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
#  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
#  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE!
#
"""
"""

__version__ = '$Id: plugin_ztxtoutput.py,v 1.5 2001/12/21 06:23:13 rob Exp $'

__author__ = 'Rob Tillotson <rob@pyrite.org>'

__copyright__ = 'Copyright 2001 Rob Tillotson <rob@pyrite.org>'

import string, os

from dtkplugins import OutputPlugin
import ztxt

def valid_font(s):
    s = string.lower(s)
    if s not in ['standard','normal','large','bold','largebold','hk40','fixed']:
        raise OptionParsingError, 'font must be one of "normal", "large", "bold", "largebold", or "fixed"'
    if s == 'hk40': s == 'fixed'
    if s == 'standard': s == 'normal'
    return s

class Plugin(OutputPlugin):
    name = 'zTXT'
    description = 'Outputs to a zTXT (Gutenpalm) database.'

    links = [ (-1, "doc-output", None) ]

    def __init__(self, *a, **kw):
        OutputPlugin.__init__(self)

        self._add_property('title', 'The document title')
        self._add_cli_option('title','t','title',
                             'Set the document title',
                             vtype='STR')
        self.title = ''

        self._add_property('output_dir', 'Output directory')
        self._add_cli_option('output_dir', 'd', 'output-directory',
                             'Set the output directory',
                             vtype='STR')
        self.output_dir = '.'
        
        self._add_property('random_access', 'Compress for random access', boolean=1)
        self._add_cli_option('random_access', None, 'random-access',
                             'Toggle random access (default is on)',
                             boolean=1)
        self.random_access = 1

#         self._add_property('rotation', 'Rotation (0, 90, 180, or 270 degrees)')
#         self._add_cli_option('rotation', None, 'rotate',
#                              'Rotate display (0, 90, 180, or 270 degrees)',
#                              vtype='NUM', func=int)
#         self.rotation = 0

#         self._add_property('font', 'Font name')
#         self._add_cli_option('font', 'F', 'font',
#                              'Font name (normal, bold, large, largebold, fixed)',
#                              vtype='NAME', func=valid_font)
#         self.font = 'normal'

#         self._add_property('justify', 'Justify text')
#         self._add_cli_option('justify','j', 'justify',
#                              'Justify text',
#                              boolean=1)
#         self.justify = 0

#         self._add_property('fullscreen', 'Full screen display')
#         self._add_cli_option('fullscreen', None, 'fullscreen',
#                              'Full screen display',
#                              boolean=1)
#         self.fullscreen = 0

#         self._add_property('invert', 'Invert the display')
#         self._add_cli_option('invert', None, 'invert',
#                              'Invert the display',
#                              boolean=1)
#         self.invert = 0
            
        self._add_property('output_filename', 'Output filename')
        self._add_cli_option('output_filename', 'o', None,
                             'Set the output filename',
                             vtype="STR")
        self.output_filename = ''
        
        self.doc = None

    def open(self, chain, next, basename, *a, **kw):
        params = 0
#         if self.invert: params = params | 0x0001
#         if self.fullscreen: params = params | 0x0002
#         if self.justify: params = params | 0x0004
        
#         if self.rotation == 0: params = params | 0x0010
#         elif self.rotation == 90: params = params | 0x0020
#         elif self.rotation == 180: params = params | 0x0040
#         elif self.rotation == 270: params = params | 0x0080

#         if self.font == 'normal': params = params | 0x0100
#         elif self.font == 'large': params = params | 0x0200
#         elif self.font == 'bold': params = params | 0x0400
#         elif self.font == 'largebold': params = params | 0x0800
#         elif self.font == 'fixed': params = params | 0x1000

        if self.output_filename: fn = self.output_filename
        else: fn = os.path.join(self.output_dir, basename+'.pdb')
        
        self.doc = ztxt.zTXTWriteStream(fn,
                                        self.title and self.title or basename,
                                        self.random_access, params)
        return self.doc

    def close(self):
        self.doc = None
        
